@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2005 Martyn Russell <mr@gnome.org>
+ * Copyright (C) 2005-2007 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,16 +18,17 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
+#include "config.h"
+
 #include <string.h>
 
 #include <glib.h>
 #include <glib/gi18n.h>
 
-#include "libgossip-marshal.h"
-
 #include "gossip-chatroom.h"
 
+#include "libgossip-marshal.h"
+
 #define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GOSSIP_TYPE_CHATROOM, GossipChatroomPriv))
 
 typedef struct _GossipChatroomPriv GossipChatroomPriv;
@@ -51,6 +52,8 @@ struct _GossipChatroomPriv {
 
 	GossipChatroomStatus  status;
 	gchar                *last_error;
+
+	GHashTable           *contacts;
 };
 
 struct _GossipChatroomInvite {
@@ -91,7 +94,9 @@ enum {
 };
 
 enum {
-	CHANGED,
+	CONTACT_JOINED,
+	CONTACT_LEFT,
+	CONTACT_INFO_CHANGED,
 	LAST_SIGNAL
 };
 
@@ -125,6 +130,20 @@ gossip_chatroom_get_gtype (void)
 	return type;
 }
 
+GType
+gossip_chatroom_invite_get_gtype (void)
+{
+	static GType type_id = 0;
+
+	if (!type_id) {
+		type_id = g_boxed_type_register_static ("GossipChatroomInvite",
+							(GBoxedCopyFunc) gossip_chatroom_invite_ref,
+							(GBoxedFreeFunc) gossip_chatroom_invite_unref);
+	}
+
+	return type_id;
+}
+
 static void
 chatroom_class_init (GossipChatroomClass *class)
 {
@@ -255,16 +274,33 @@ chatroom_class_init (GossipChatroomClass *class)
 	 * signals
 	 */
 
-	signals[CHANGED] =
-		g_signal_new ("changed",
+	signals[CONTACT_JOINED] =
+		g_signal_new ("contact-joined",
 			      G_TYPE_FROM_CLASS (object_class),
 			      G_SIGNAL_RUN_LAST,
 			      0,
 			      NULL, NULL,
-			      libgossip_marshal_VOID__VOID,
+			      libgossip_marshal_VOID__OBJECT,
 			      G_TYPE_NONE,
-			      0);
-
+			      1, GOSSIP_TYPE_CONTACT);
+	signals[CONTACT_LEFT] =
+		g_signal_new ("contact-left",
+			      G_TYPE_FROM_CLASS (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      0,
+			      NULL, NULL,
+			      libgossip_marshal_VOID__OBJECT,
+			      G_TYPE_NONE,
+			      1, GOSSIP_TYPE_CONTACT);
+	signals[CONTACT_INFO_CHANGED] =
+		g_signal_new ("contact-info-changed",
+			      G_TYPE_FROM_CLASS (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      0,
+			      NULL, NULL,
+			      libgossip_marshal_VOID__OBJECT,
+			      G_TYPE_NONE,
+			      1, GOSSIP_TYPE_CONTACT);
 
 	g_type_class_add_private (object_class, sizeof (GossipChatroomPriv));
 }
@@ -295,6 +331,10 @@ chatroom_init (GossipChatroom *chatroom)
 	priv->last_error   = NULL;
 
 	priv->account      = NULL;
+	priv->contacts     = g_hash_table_new_full (gossip_contact_hash,
+						    gossip_contact_equal,
+						    (GDestroyNotify) g_object_unref,
+						    g_free);
 }
 
 static void
@@ -318,6 +358,8 @@ chatroom_finalize (GObject *object)
 		g_object_unref (priv->account);
 	}
 
+	g_hash_table_destroy (priv->contacts);
+
 	(G_OBJECT_CLASS (parent_class)->finalize) (object);
 }
 
@@ -390,7 +432,6 @@ chatroom_set_property (GObject      *object,
 	switch (param_id) {
 	case PROP_TYPE:
 		priv->type = g_value_get_int (value);
-		g_signal_emit (GOSSIP_CHATROOM (object), signals[CHANGED], 0);
 		break;
 	case PROP_NAME:
 		gossip_chatroom_set_name (GOSSIP_CHATROOM (object),
@@ -469,7 +510,7 @@ gossip_chatroom_get_id_str (GossipChatroom *chatroom)
 
 	priv = GET_PRIV (chatroom);
 
-	if (!priv->room && !priv->server) {
+	if (!priv->id_str) {
 		priv->id_str = g_strdup ("");
 	}
 
@@ -593,6 +634,20 @@ gossip_chatroom_get_account (GossipChatroom *chatroom)
 	return priv->account;
 }
 
+GossipChatroomContactInfo *
+gossip_chatroom_get_contact_info (GossipChatroom *chatroom,
+				  GossipContact  *contact)
+{
+	GossipChatroomPriv *priv;
+
+	g_return_val_if_fail (GOSSIP_IS_CHATROOM (chatroom), NULL);
+	g_return_val_if_fail (GOSSIP_IS_CONTACT (contact), NULL);
+
+	priv = GET_PRIV (chatroom);
+
+	return g_hash_table_lookup (priv->contacts, contact);
+}
+
 void
 gossip_chatroom_set_name (GossipChatroom *chatroom,
 			  const gchar    *name)
@@ -608,7 +663,6 @@ gossip_chatroom_set_name (GossipChatroom *chatroom,
 	priv->name = g_strdup (name);
 
 	g_object_notify (G_OBJECT (chatroom), "name");
-	g_signal_emit (chatroom, signals[CHANGED], 0);
 }
 
 void
@@ -626,7 +680,6 @@ gossip_chatroom_set_nick (GossipChatroom *chatroom,
 	priv->nick = g_strdup (nick);
 
 	g_object_notify (G_OBJECT (chatroom), "nick");
-	g_signal_emit (chatroom, signals[CHANGED], 0);
 }
 
 void
@@ -649,7 +702,6 @@ gossip_chatroom_set_server (GossipChatroom *chatroom,
 	}
 
 	g_object_notify (G_OBJECT (chatroom), "server");
-	g_signal_emit (chatroom, signals[CHANGED], 0);
 }
 
 void
@@ -672,7 +724,6 @@ gossip_chatroom_set_room (GossipChatroom *chatroom,
 	}
 
 	g_object_notify (G_OBJECT (chatroom), "room");
-	g_signal_emit (chatroom, signals[CHANGED], 0);
 }
 
 void
@@ -690,7 +741,6 @@ gossip_chatroom_set_password (GossipChatroom *chatroom,
 	priv->password = g_strdup (password);
 
 	g_object_notify (G_OBJECT (chatroom), "password");
-	g_signal_emit (chatroom, signals[CHANGED], 0);
 }
 
 void
@@ -705,7 +755,6 @@ gossip_chatroom_set_auto_connect (GossipChatroom *chatroom,
 	priv->auto_connect = auto_connect;
 
 	g_object_notify (G_OBJECT (chatroom), "auto_connect");
-	g_signal_emit (chatroom, signals[CHANGED], 0);
 }
 
 void
@@ -720,7 +769,16 @@ gossip_chatroom_set_favourite (GossipChatroom *chatroom,
 	priv->favourite = favourite;
 
 	g_object_notify (G_OBJECT (chatroom), "favourite");
-	g_signal_emit (chatroom, signals[CHANGED], 0);
+}
+
+static void
+chatroom_clear_contacts (GossipChatroom *chatroom)
+{
+	GossipChatroomPriv *priv;
+
+	priv = GET_PRIV (chatroom);
+
+	g_hash_table_remove_all (priv->contacts);
 }
 
 void
@@ -735,8 +793,11 @@ gossip_chatroom_set_status (GossipChatroom       *chatroom,
 
 	priv->status = status;
 
+	if (status == GOSSIP_CHATROOM_STATUS_INACTIVE) {
+		chatroom_clear_contacts (chatroom);
+	}
+
 	g_object_notify (G_OBJECT (chatroom), "status");
-	g_signal_emit (chatroom, signals[CHANGED], 0);
 }
 
 void
@@ -757,7 +818,6 @@ gossip_chatroom_set_last_error (GossipChatroom *chatroom,
 	}
 
 	g_object_notify (G_OBJECT (chatroom), "last_error");
-	g_signal_emit (chatroom, signals[CHANGED], 0);
 }
 
 void
@@ -777,7 +837,31 @@ gossip_chatroom_set_account (GossipChatroom *chatroom,
 	priv->account = g_object_ref (account);
 
 	g_object_notify (G_OBJECT (chatroom), "account");
-	g_signal_emit (chatroom, signals[CHANGED], 0);
+}
+
+void
+gossip_chatroom_set_contact_info (GossipChatroom            *chatroom,
+				  GossipContact             *contact,
+				  GossipChatroomContactInfo *info)
+{
+	GossipChatroomPriv        *priv;
+	GossipChatroomContactInfo *chatroom_contact;
+
+	g_return_if_fail (GOSSIP_IS_CHATROOM (chatroom));
+	g_return_if_fail (GOSSIP_IS_CONTACT (contact));
+	g_return_if_fail (info != NULL);
+
+	priv = GET_PRIV (chatroom);
+
+	chatroom_contact = g_new0 (GossipChatroomContactInfo, 1);
+	chatroom_contact->role = info->role;
+	chatroom_contact->affiliation = info->affiliation;
+	
+	g_hash_table_insert (priv->contacts, 
+			     g_object_ref (contact),
+			     chatroom_contact);
+
+	g_signal_emit (chatroom, signals[CONTACT_INFO_CHANGED], 0, contact);
 }
 
 guint
@@ -805,11 +889,7 @@ gossip_chatroom_equal (gconstpointer a,
 	priv1 = GET_PRIV (a);
 	priv2 = GET_PRIV (b);
 
-	if (!priv1) {
-		return FALSE;
-	}
-
-	if (!priv2) {
+	if (!priv1 || !priv2) {
 		return FALSE;
 	}
 
@@ -829,11 +909,7 @@ gossip_chatroom_equal_full (gconstpointer a,
 	priv1 = GET_PRIV (a);
 	priv2 = GET_PRIV (b);
 
-	if (!priv1) {
-		return FALSE;
-	}
-
-	if (!priv2) {
+	if (!priv1 || !priv2) {
 		return FALSE;
 	}
 
@@ -943,6 +1019,53 @@ gossip_chatroom_affiliation_to_string (GossipChatroomAffiliation affiliation,
 	return "";
 }
 
+void
+gossip_chatroom_contact_joined (GossipChatroom            *chatroom,
+				GossipContact             *contact,
+				GossipChatroomContactInfo *info)
+{
+	GossipChatroomPriv        *priv;
+	GossipChatroomContactInfo *chatroom_contact;
+
+	g_return_if_fail (GOSSIP_IS_CHATROOM (chatroom));
+	g_return_if_fail (GOSSIP_IS_CONTACT (contact));
+
+	priv = GET_PRIV (chatroom);
+
+	chatroom_contact = g_new0 (GossipChatroomContactInfo, 1);
+	if (info) {
+		chatroom_contact->role = info->role;
+		chatroom_contact->affiliation = info->affiliation;		
+	} else {
+		chatroom_contact->role = GOSSIP_CHATROOM_ROLE_NONE;
+		chatroom_contact->affiliation = GOSSIP_CHATROOM_AFFILIATION_NONE;
+	}
+	if (!g_hash_table_lookup (priv->contacts, contact)) {
+		g_hash_table_insert (priv->contacts,
+				     g_object_ref (contact),
+				     chatroom_contact);
+
+		g_signal_emit (chatroom, signals[CONTACT_JOINED], 0, contact);
+	}
+}
+
+void
+gossip_chatroom_contact_left (GossipChatroom *chatroom,
+			      GossipContact  *contact)
+{
+	GossipChatroomPriv *priv;
+
+	g_return_if_fail (GOSSIP_IS_CHATROOM (chatroom));
+	g_return_if_fail (GOSSIP_IS_CONTACT (contact));
+
+	priv = GET_PRIV (chatroom);
+
+	if (g_hash_table_lookup (priv->contacts, contact)) {
+		g_signal_emit (chatroom, signals[CONTACT_LEFT], 0, contact);
+		g_hash_table_remove (priv->contacts, contact);
+	}
+}
+
 /*
  * Chatroom invite functions
  */
@@ -1041,81 +1164,3 @@ gossip_chatroom_invite_get_reason (GossipChatroomInvite *invite)
 	return invite->reason;
 }
 
-GType
-gossip_chatroom_role_get_gtype (void)
-{
-	static GType type = 0;
-
-	if (type == 0) {
-		static const GEnumValue values[] = {
-			{
-				GOSSIP_CHATROOM_ROLE_MODERATOR,
-				"GOSSIP_CHATROOM_ROLE_MODERATOR",
-				"Moderator"
-			},
-			{
-				GOSSIP_CHATROOM_ROLE_PARTICIPANT,
-				"GOSSIP_CHATROOM_ROLE_PARTICIPANT",
-				"Participant"
-			},
-			{
-				GOSSIP_CHATROOM_ROLE_VISITOR,
-				"GOSSIP_CHATROOM_ROLE_VISITOR",
-				"Visitor"
-			},
-			{
-				GOSSIP_CHATROOM_ROLE_NONE,
-				"GOSSIP_CHATROOM_ROLE_NONE",
-				"None"
-			},
-			{ 0, NULL, NULL }
-		};
-
-		type = g_enum_register_static ("GossipChatroomRole", values);
-	}
-
-	return type;
-}
-
-GType
-gossip_chatroom_affiliation_get_gtype (void)
-{
-	static GType type = 0;
-
-	if (type == 0) {
-		static const GEnumValue values[] = {
-			{
-				GOSSIP_CHATROOM_AFFILIATION_OWNER,
-				"GOSSIP_CHATROOM_AFFILIATION_OWNER",
-				"Owner"
-			},
-			{
-				GOSSIP_CHATROOM_AFFILIATION_ADMIN,
-				"GOSSIP_CHATROOM_AFFILIATION_ADMIN",
-				"Admin"
-			},
-			{
-				GOSSIP_CHATROOM_AFFILIATION_MEMBER,
-				"GOSSIP_CHATROOM_AFFILIATION_MEMBER",
-				"Member"
-			},
-			{
-				GOSSIP_CHATROOM_AFFILIATION_OUTCAST,
-				"GOSSIP_CHATROOM_AFFILIATION_OUTCAST",
-				"Outcast"
-			},
-			{
-				GOSSIP_CHATROOM_AFFILIATION_NONE,
-				"GOSSIP_CHATROOM_AFFILIATION_NONE",
-				"None"
-			},
-			{ 0, NULL, NULL }
-		};
-
-		type = g_enum_register_static ("GossipChatroomAffiliation",
-					       values);
-	}
-
-	return type;
-}
-
