@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2004 Imendio AB
+ * Copyright (C) 2004-2007 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,15 +16,17 @@
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
+ *
+ * Authors: Mikael Hallendal <micke@imendio.com>
+ *          Martyn Russell <martyn@imendio.com>
  */
 
-#include <config.h>
+#include "config.h"
 
-#include <glib/gi18n.h>
 #include <string.h>
 
-#include "gossip-account.h"
-#include "gossip-avatar.h"
+#include <glib/gi18n.h>
+
 #include "gossip-contact.h"
 #include "gossip-utils.h"
 
@@ -169,10 +171,11 @@ contact_class_init (GossipContactClass *class)
 
 	g_object_class_install_property (object_class,
 					 PROP_AVATAR,
-					 g_param_spec_pointer ("avatar",
-							       "Avatar image",
-							       "The avatar image",
-							       G_PARAM_READWRITE));
+					 g_param_spec_boxed ("avatar",
+							     "Avatar image",
+							     "The avatar image",
+							     GOSSIP_TYPE_AVATAR,
+							     G_PARAM_READWRITE));
 
 	g_object_class_install_property (object_class,
 					 PROP_ACCOUNT,
@@ -264,14 +267,10 @@ contact_get_property (GObject    *object,
 		g_value_set_int (value, priv->subscription);
 		break;
 	case PROP_AVATAR:
-		g_value_set_pointer (value, priv->avatar);
+		g_value_set_boxed (value, priv->avatar);
 		break;
 	case PROP_ACCOUNT:
-		if (priv->account) {
-			g_value_set_object (value, G_OBJECT (priv->account));
-		} else {
-			g_value_set_object (value, NULL);
-		}
+		g_value_set_object (value, priv->account);
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
@@ -316,7 +315,7 @@ contact_set_property (GObject      *object,
 		break;
 	case PROP_AVATAR:
 		gossip_contact_set_avatar (GOSSIP_CONTACT (object),
-					   g_value_get_pointer (value));
+					   g_value_get_boxed (value));
 		break;
 	case PROP_ACCOUNT:
 		gossip_contact_set_account (GOSSIP_CONTACT (object),
@@ -440,6 +439,22 @@ gossip_contact_get_avatar (GossipContact *contact)
 	return priv->avatar;
 }
 
+GdkPixbuf *
+gossip_contact_get_avatar_pixbuf (GossipContact *contact)
+{
+	GossipContactPriv *priv;
+
+	g_return_val_if_fail (GOSSIP_IS_CONTACT (contact), NULL);
+
+	priv = GET_PRIV (contact);
+
+	if (!priv->avatar) {
+		return NULL;
+	}
+
+	return gossip_avatar_get_pixbuf (priv->avatar);
+}
+
 GossipAccount *
 gossip_contact_get_account (GossipContact *contact)
 {
@@ -610,7 +625,7 @@ gossip_contact_set_avatar (GossipContact *contact,
 		gossip_avatar_unref (priv->avatar);
 		priv->avatar = NULL;
 	}
-	
+
 	if (avatar) {
 		priv->avatar = gossip_avatar_ref (avatar);
 	}
@@ -683,7 +698,7 @@ gossip_contact_add_presence (GossipContact  *contact,
 	}
 
 	/* Add new presence */
-	priv->presences = g_list_insert_sorted (priv->presences, 
+	priv->presences = g_list_insert_sorted (priv->presences,
 						g_object_ref (presence),
 						gossip_presence_sort_func);
 
@@ -817,38 +832,38 @@ gboolean
 gossip_contact_equal (gconstpointer v1,
 		      gconstpointer v2)
 {
-	const gchar   *id_a;
-	const gchar   *id_b;
 	GossipAccount *account_a;
 	GossipAccount *account_b;
+	const gchar   *id_a;
+	const gchar   *id_b;
 
 	g_return_val_if_fail (GOSSIP_IS_CONTACT (v1), FALSE);
 	g_return_val_if_fail (GOSSIP_IS_CONTACT (v2), FALSE);
 
-	id_a = gossip_contact_get_id (GOSSIP_CONTACT (v1));
-	id_b = gossip_contact_get_id (GOSSIP_CONTACT (v2));
-
 	account_a = gossip_contact_get_account (GOSSIP_CONTACT (v1));
 	account_b = gossip_contact_get_account (GOSSIP_CONTACT (v2));
 
-	return g_str_equal (id_a, id_b) &&
-	       gossip_account_equal (account_a, account_b);
+	id_a = gossip_contact_get_id (GOSSIP_CONTACT (v1));
+	id_b = gossip_contact_get_id (GOSSIP_CONTACT (v2));
+
+	return gossip_account_equal (account_a, account_b) && g_str_equal (id_a, id_b);
 }
 
 guint
 gossip_contact_hash (gconstpointer key)
 {
 	GossipContactPriv *priv;
+	guint              hash = 0;
 
 	g_return_val_if_fail (GOSSIP_IS_CONTACT (key), +1);
 
 	priv = GET_PRIV (GOSSIP_CONTACT (key));
 
-	return g_str_hash (gossip_contact_get_id (GOSSIP_CONTACT (key))) + 
-	       gossip_account_hash (gossip_contact_get_account (GOSSIP_CONTACT (key)));
-}
+	hash += gossip_account_hash (gossip_contact_get_account (GOSSIP_CONTACT (key)));
+	hash += g_str_hash (gossip_contact_get_id (GOSSIP_CONTACT (key)));
 
-/* convenience functions */
+	return hash;
+}
 
 gboolean
 gossip_contact_is_online (GossipContact *contact)
@@ -866,6 +881,24 @@ gossip_contact_is_online (GossipContact *contact)
 	return TRUE;
 }
 
+gboolean
+gossip_contact_is_in_group (GossipContact *contact,
+			    const gchar   *group)
+{
+	GossipContactPriv *priv;
+
+	g_return_val_if_fail (GOSSIP_IS_CONTACT (contact), FALSE);
+	g_return_val_if_fail (!G_STR_EMPTY (group), FALSE);
+
+	priv = GET_PRIV (contact);
+
+	if (g_list_find_custom (priv->groups, group, (GCompareFunc) strcmp)) {
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
 const gchar *
 gossip_contact_get_status (GossipContact *contact)
 {
