@@ -22,11 +22,11 @@
 #define __GOSSIP_CONTACT_H__
 
 #include <glib-object.h>
+#include <gdk-pixbuf/gdk-pixbuf.h>
 
 #include "gossip-account.h"
 #include "gossip-avatar.h"
 #include "gossip-presence.h"
-#include "gossip-utils.h"
 
 G_BEGIN_DECLS
 
@@ -44,7 +44,6 @@ struct _GossipContact {
 	GObject parent;
 };
 
-
 struct _GossipContactClass {
 	GObjectClass parent_class;
 };
@@ -53,7 +52,7 @@ typedef enum {
 	GOSSIP_CONTACT_TYPE_TEMPORARY,
 	GOSSIP_CONTACT_TYPE_CONTACTLIST,
 	GOSSIP_CONTACT_TYPE_CHATROOM,
-	GOSSIP_CONTACT_TYPE_USER /* Represents the own user */
+	GOSSIP_CONTACT_TYPE_USER           /* Represents the own user */
 } GossipContactType;
 
 typedef enum {
@@ -78,6 +77,7 @@ GossipContactType  gossip_contact_get_type                  (GossipContact
 const gchar *      gossip_contact_get_id                    (GossipContact      *contact);
 const gchar *      gossip_contact_get_name                  (GossipContact      *contact);
 GossipAvatar *     gossip_contact_get_avatar                (GossipContact      *contact);
+GdkPixbuf *        gossip_contact_get_avatar_pixbuf         (GossipContact      *contact);
 GossipAccount *    gossip_contact_get_account               (GossipContact      *contact);
 void               gossip_contact_add_presence              (GossipContact      *contact,
 							     GossipPresence     *presence);
@@ -112,6 +112,8 @@ gboolean           gossip_contact_equal                     (gconstpointer
 							     gconstpointer       v2);
 guint              gossip_contact_hash                      (gconstpointer       key);
 gboolean           gossip_contact_is_online                 (GossipContact      *contact);
+gboolean           gossip_contact_is_in_group               (GossipContact      *contact,
+							     const gchar        *group);
 const gchar *      gossip_contact_get_status                (GossipContact      *contact);
 
 G_END_DECLS
