@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2002-2006 Imendio AB
+ * Copyright (C) 2002-2007 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,17 +18,24 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
+#include "config.h"
+
 #include <string.h>
+
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 #include <glade/glade.h>
 #include <glib/gi18n.h>
 
-#include <libgossip/gossip-chatroom-contact.h>
-#include <libgossip/gossip-chatroom-provider.h>
-#include <libgossip/gossip-debug.h>
+#include <libgossip/gossip-stock.h>
+
+#include <libgossip/gossip-contact.h>
+#include <libgossip/gossip-session.h>
+#include <libgossip/gossip-protocol.h>
+#include <libgossip/gossip-utils.h>
+#include <libgossip/gossip-account.h>
 #include <libgossip/gossip-log.h>
+#include <libgossip/gossip-debug.h>
 #include <libgossip/gossip-message.h>
 
 #include "gossip-app.h"
@@ -38,17 +45,17 @@
 #include "gossip-chat-invite.h"
 #include "gossip-chat-view.h"
 #include "gossip-contact-list-iface.h"
+#include "gossip-glade.h"
 #include "gossip-group-chat.h"
 #include "gossip-private-chat.h"
 #include "gossip-sound.h"
-#include "gossip-stock.h"
 #include "gossip-ui-utils.h"
 
-#define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GOSSIP_TYPE_GROUP_CHAT, GossipGroupChatPriv))
+#define DEBUG_DOMAIN "GroupChat"
 
 #define IS_ENTER(v) (v == GDK_Return || v == GDK_ISO_Enter || v == GDK_KP_Enter)
 
-#define DEBUG_DOMAIN "GroupChat"
+#define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GOSSIP_TYPE_GROUP_CHAT, GossipGroupChatPriv))
 
 struct _GossipGroupChatPriv {
 	GossipContact          *own_contact;
@@ -81,6 +88,8 @@ struct _GossipGroupChatPriv {
 
 	gint                    contacts_width;
 	gboolean                contacts_visible;
+
+	GossipTime              time_joined;
 };
 
 typedef struct {
@@ -174,14 +183,19 @@ static void            group_chat_topic_entry_activate_cb     (GtkWidget
 static void            group_chat_topic_response_cb           (GtkWidget                    *dialog,
 							       gint                          response,
 							       GossipGroupChat              *chat);
-static void            group_chat_contact_joined_cb           (GossipChatroomProvider       *provider,
-							       gint                          id,
+static void            group_chat_contact_joined_cb           (GossipChatroom               *chatroom,
 							       GossipContact                *contact,
 							       GossipGroupChat              *chat);
-static void            group_chat_contact_left_cb             (GossipChatroomProvider       *provider,
-							       gint                          id,
+static void            group_chat_contact_left_cb             (GossipChatroom               *chatroom,
 							       GossipContact                *contact,
 							       GossipGroupChat              *chat);
+static void            group_chat_contact_info_changed_cb     (GossipChatroom               *chatroom,
+							       GossipContact                *contact,
+							       GossipGroupChat              *chat);
+static void            group_chat_contact_add                 (GossipGroupChat              *chat,
+							       GossipContact                *contact);
+static void            group_chat_contact_remove              (GossipGroupChat              *chat,
+							       GossipContact                *contact);
 static void            group_chat_contact_presence_updated_cb (GossipContact                *contact,
 							       GParamSpec                   *param,
 							       GossipGroupChat              *chat);
@@ -360,12 +374,15 @@ group_chat_finalize (GObject *object)
 	g_signal_handlers_disconnect_by_func (priv->chatroom_provider,
 					      group_chat_topic_changed_cb,
 					      chat);
-	g_signal_handlers_disconnect_by_func (priv->chatroom_provider,
+	g_signal_handlers_disconnect_by_func (priv->chatroom,
 					      group_chat_contact_joined_cb,
 					      chat);
-	g_signal_handlers_disconnect_by_func (priv->chatroom_provider,
+	g_signal_handlers_disconnect_by_func (priv->chatroom,
 					      group_chat_contact_left_cb,
 					      chat);
+	g_signal_handlers_disconnect_by_func (priv->chatroom,
+					      group_chat_contact_info_changed_cb,
+					      chat);
 	
 	g_object_unref (priv->chatroom_provider);
 
@@ -563,6 +580,27 @@ group_chat_key_press_event_cb (GtkWidget       *widget,
 
 	priv = GET_PRIV (chat);
 
+	/* Catch ctrl+up/down so we can traverse messages we sent */
+	if ((event->state & GDK_CONTROL_MASK) && 
+	    (event->keyval == GDK_Up || 
+	     event->keyval == GDK_Down)) {
+		GtkTextBuffer *buffer;
+		const gchar   *str;
+
+		buffer = gtk_text_view_get_buffer 
+			(GTK_TEXT_VIEW (GOSSIP_CHAT (chat)->input_text_view));
+
+		if (event->keyval == GDK_Up) {
+			str = gossip_chat_sent_message_get_next (GOSSIP_CHAT (chat));
+		} else {
+			str = gossip_chat_sent_message_get_last (GOSSIP_CHAT (chat));
+		}
+
+		gtk_text_buffer_set_text (buffer, str ? str : "", -1);
+
+		return TRUE;    
+	}
+
 	/* Catch enter but not ctrl/shift-enter */
 	if (IS_ENTER (event->keyval) && !(event->state & GDK_SHIFT_MASK)) {
 		GossipChat   *parent;
@@ -1084,6 +1122,9 @@ group_chat_new_message_cb (GossipChatroomProvider *provider,
 {
 	GossipGroupChatPriv  *priv;
 	GossipChatroomInvite *invite;
+	GossipLogManager     *log_manager;
+	GossipTime            timestamp;
+	gboolean              is_backlog;
 
 	priv = GET_PRIV (chat);
 
@@ -1091,7 +1132,12 @@ group_chat_new_message_cb (GossipChatroomProvider *provider,
 		return;
 	}
 
-	gossip_debug (DEBUG_DOMAIN, "[%d] New message", id);
+	timestamp = gossip_message_get_timestamp (message);
+	is_backlog = timestamp < priv->time_joined;
+
+	gossip_debug (DEBUG_DOMAIN, 
+		      "[%d] New message with timestamp:%d, message %s backlog", 
+		      id, timestamp, is_backlog ? "IS" : "IS NOT");
 
 	invite = gossip_message_get_invite (message);
 	if (invite) {
@@ -1116,14 +1162,18 @@ group_chat_new_message_cb (GossipChatroomProvider *provider,
 		}
 	}
 
-	if (gossip_chat_should_play_sound (GOSSIP_CHAT (chat)) &&
+	if (!is_backlog && 
+	    gossip_chat_should_play_sound (GOSSIP_CHAT (chat)) &&
 	    gossip_chat_should_highlight_nick (message, priv->own_contact)) {
 		gossip_sound_play (GOSSIP_SOUND_CHAT);
 	}
 
-	gossip_log_message_for_chatroom (priv->chatroom, message, FALSE);
+	if (!is_backlog) {
+		log_manager = gossip_session_get_log_manager (gossip_app_get_session ());
+		gossip_log_message_for_chatroom (log_manager, priv->chatroom, message, FALSE);
+	}
 
-	g_signal_emit_by_name (chat, "new-message", message);
+	g_signal_emit_by_name (chat, "new-message", message, is_backlog);
 }
 
 static void
@@ -1215,14 +1265,20 @@ void
 gossip_group_chat_set_topic (GossipGroupChat *chat)
 {
 	GossipGroupChatPriv *priv;
+	GossipChatWindow    *chat_window;
+	GtkWidget           *chat_dialog;
 	GtkWidget           *dialog;
 	GtkWidget           *entry;
 	GtkWidget           *hbox;
 	const gchar         *topic;
 
+
 	priv = GET_PRIV (chat);
 
-	dialog = gtk_message_dialog_new (GTK_WINDOW (gossip_app_get_window ()),
+	chat_window = gossip_chat_get_window (GOSSIP_CHAT (chat));
+	chat_dialog = gossip_chat_window_get_dialog (chat_window);
+
+	dialog = gtk_message_dialog_new (GTK_WINDOW (chat_dialog),
 					 0,
 					 GTK_MESSAGE_QUESTION,
 					 GTK_BUTTONS_OK_CANCEL,
@@ -1253,71 +1309,6 @@ gossip_group_chat_set_topic (GossipGroupChat *chat)
 	gtk_widget_show_all (dialog);
 }
 
-static void
-group_chat_contact_joined_cb (GossipChatroomProvider *provider,
-			      gint                    id,
-			      GossipContact          *contact,
-			      GossipGroupChat        *chat)
-{
-	GossipGroupChatPriv *priv;
-	GtkTreeModel        *model;
-	GtkTreeIter          iter;
-	GtkTreeIter          parent;
-	GdkPixbuf           *pixbuf;
-	GossipChatroomRole   role;
-	GtkTreePath         *path;
-
-	priv = GET_PRIV (chat);
-
-	if (id != gossip_chatroom_get_id (priv->chatroom)) {
-		return;
-	}
-
-	gossip_debug (DEBUG_DOMAIN, "[%d] Contact joined:'%s'",
-		      id, gossip_contact_get_id (contact));
-
-	pixbuf = gossip_pixbuf_for_contact (contact);
-
-	role = gossip_chatroom_contact_get_role (GOSSIP_CHATROOM_CONTACT (contact));
-	group_chat_get_role_iter (chat, role, &parent);
-
-	model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->treeview));
-	gtk_tree_store_append (GTK_TREE_STORE (model), &iter, &parent);
-
-	gtk_tree_store_set (GTK_TREE_STORE (model),
-			    &iter,
-			    COL_CONTACT, contact,
-			    COL_NAME, gossip_contact_get_name (contact),
-			    COL_STATUS, pixbuf,
-			    COL_IS_HEADER, FALSE,
-			    -1);
-
-	path = gtk_tree_model_get_path (model, &parent);
-	gtk_tree_view_expand_row (GTK_TREE_VIEW (priv->treeview), path, TRUE);
-	gtk_tree_path_free (path);
-
-	g_object_unref (pixbuf);
-
-	g_signal_connect (contact, "notify::presences",
-			  G_CALLBACK (group_chat_contact_presence_updated_cb),
-			  chat);
-	g_signal_connect (contact, "notify::name",
-			  G_CALLBACK (group_chat_contact_updated_cb),
-			  chat);
-
-	g_signal_emit_by_name (chat, "contact_added", contact);
-	
-	/* Add event to chatroom */
-	if (!gossip_contact_equal (priv->own_contact, contact)) {
-		gchar *str;
-
-		str = g_strdup_printf (_("%s has joined the room"),
-				       gossip_contact_get_name (contact));
-		gossip_chat_view_append_event (GOSSIP_CHAT (chat)->view, str);
-		g_free (str);
-	}
-}
-
 static void
 group_chat_cl_pixbuf_cell_data_func (GtkTreeViewColumn *tree_column,
 				     GtkCellRenderer   *cell,
@@ -1450,22 +1441,52 @@ group_chat_cl_set_background (GossipGroupChat *chat,
 }
 
 static void
-group_chat_contact_left_cb (GossipChatroomProvider *provider,
-			    gint                    id,
-			    GossipContact          *contact,
-			    GossipGroupChat        *chat)
+group_chat_contact_joined_cb (GossipChatroom  *chatroom,
+			      GossipContact   *contact,
+			      GossipGroupChat *chat)
 {
-	GossipGroupChatPriv *priv;
-	GtkTreeIter          iter;
+	GossipGroupChatPriv       *priv;
 
 	priv = GET_PRIV (chat);
 
-	if (id != gossip_chatroom_get_id (priv->chatroom)) {
-		return;
+	gossip_debug (DEBUG_DOMAIN, "Contact joined:'%s'",
+		      gossip_contact_get_id (contact));
+
+	group_chat_contact_add (chat, contact);
+
+	g_signal_connect (contact, "notify::presences",
+			  G_CALLBACK (group_chat_contact_presence_updated_cb),
+			  chat);
+	g_signal_connect (contact, "notify::name",
+			  G_CALLBACK (group_chat_contact_updated_cb),
+			  chat);
+
+	g_signal_emit_by_name (chat, "contact_added", contact);
+	
+	/* Add event to chatroom */
+	if (!gossip_contact_equal (priv->own_contact, contact)) {
+		gchar *str;
+
+		str = g_strdup_printf (_("%s has joined the room"),
+				       gossip_contact_get_name (contact));
+		gossip_chat_view_append_event (GOSSIP_CHAT (chat)->view, str);
+		g_free (str);
 	}
 
-	gossip_debug (DEBUG_DOMAIN, "[%d] Contact left:'%s'",
-		      id, gossip_contact_get_id (contact));
+	priv->time_joined = gossip_time_get_current ();
+}
+
+static void
+group_chat_contact_left_cb (GossipChatroom  *chatroom,
+			    GossipContact   *contact,
+			    GossipGroupChat *chat)
+{
+	GossipGroupChatPriv *priv;
+
+	priv = GET_PRIV (chat);
+
+	gossip_debug (DEBUG_DOMAIN, "Contact left:'%s'",
+		      gossip_contact_get_id (contact));
 
 	g_signal_handlers_disconnect_by_func (contact,
 					      group_chat_contact_updated_cb,
@@ -1474,6 +1495,79 @@ group_chat_contact_left_cb (GossipChatroomProvider *provider,
 					      group_chat_contact_presence_updated_cb,
 					      chat);
 
+	group_chat_contact_remove (chat, contact);
+
+	g_signal_emit_by_name (chat, "contact_removed", contact);
+
+	/* Add event to chatroom */
+	if (!gossip_contact_equal (priv->own_contact, contact)) {
+		gchar *str;
+		str = g_strdup_printf (_("%s has left the room"),
+				       gossip_contact_get_name (contact));
+		gossip_chat_view_append_event (GOSSIP_CHAT (chat)->view, str);
+		g_free (str);
+	}
+}
+
+static void
+group_chat_contact_info_changed_cb (GossipChatroom            *chatroom,
+				    GossipContact             *contact,
+				    GossipGroupChat           *chat)
+{
+	group_chat_contact_remove (chat, contact);
+	group_chat_contact_add (chat, contact);
+}
+
+static void
+group_chat_contact_add (GossipGroupChat *chat,
+			GossipContact   *contact)
+{
+	GossipGroupChatPriv       *priv;
+	GossipChatroomContactInfo *info;
+	GossipChatroomRole         role = GOSSIP_CHATROOM_ROLE_NONE;
+	GtkTreeModel              *model;
+	GtkTreeIter                iter;
+	GtkTreeIter                parent;
+	GdkPixbuf                 *pixbuf;
+	GtkTreePath               *path;
+
+	priv = GET_PRIV (chat);
+
+	pixbuf = gossip_pixbuf_for_contact (contact);
+
+	info = gossip_chatroom_get_contact_info (priv->chatroom, contact);
+	if (info) {
+		role = info->role;
+	}
+	group_chat_get_role_iter (chat, role, &parent);
+
+	model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->treeview));
+	gtk_tree_store_append (GTK_TREE_STORE (model), &iter, &parent);
+
+	gtk_tree_store_set (GTK_TREE_STORE (model),
+			    &iter,
+			    COL_CONTACT, contact,
+			    COL_NAME, gossip_contact_get_name (contact),
+			    COL_STATUS, pixbuf,
+			    COL_IS_HEADER, FALSE,
+			    -1);
+
+	path = gtk_tree_model_get_path (model, &parent);
+	gtk_tree_view_expand_row (GTK_TREE_VIEW (priv->treeview), path, TRUE);
+	gtk_tree_path_free (path);
+
+	g_object_unref (pixbuf);
+}
+
+static void
+group_chat_contact_remove (GossipGroupChat *chat,
+			   GossipContact   *contact)
+{
+	GossipGroupChatPriv *priv;
+	GtkTreeIter          iter;
+
+	priv = GET_PRIV (chat);
+
 	if (group_chat_contacts_find (chat, contact, &iter)) {
 		GtkTreeModel *model;
 		GtkTreeIter   parent;
@@ -1486,17 +1580,6 @@ group_chat_contact_left_cb (GossipChatroomProvider *provider,
 		if (!gtk_tree_model_iter_has_child (model, &parent)) {
 			gtk_tree_store_remove (GTK_TREE_STORE (model), &parent);
 		}
-
-		g_signal_emit_by_name (chat, "contact_removed", contact);
-
-		/* Add event to chatroom */
-		if (!gossip_contact_equal (priv->own_contact, contact)) {
-			gchar *str;
-			str = g_strdup_printf (_("%s has left the room"),
-					       gossip_contact_get_name (contact));
-			gossip_chat_view_append_event (GOSSIP_CHAT (chat)->view, str);
-			g_free (str);
-		}
 	}
 }
 
@@ -1629,7 +1712,7 @@ group_chat_send (GossipGroupChat *chat)
 	gtk_text_buffer_get_bounds (buffer, &start, &end);
 	msg = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
 
-	if (msg == NULL || msg[0] == '\0') {
+	if (G_STR_EMPTY (msg)) {
 		g_free (msg);
 		return;
 	}
@@ -1669,6 +1752,8 @@ group_chat_send (GossipGroupChat *chat)
 		gossip_chatroom_provider_send (priv->chatroom_provider, 
 					       gossip_chatroom_get_id (priv->chatroom), 
 					       msg);
+		
+		gossip_chat_sent_message_add (GOSSIP_CHAT (chat), msg);
 
 		gossip_app_set_not_away ();
 	}
@@ -1788,8 +1873,9 @@ group_chat_get_status_pixbuf (GossipChat *chat)
 	}
 
 	if (!pixbuf) {
-		pixbuf = gossip_pixbuf_from_stock (GOSSIP_STOCK_GROUP_MESSAGE,
-						   GTK_ICON_SIZE_MENU);
+		pixbuf = gossip_stock_create_pixbuf (gossip_app_get_window (),
+						     GOSSIP_STOCK_GROUP_MESSAGE,
+						     GTK_ICON_SIZE_MENU);
 	}
 
 	return pixbuf;
@@ -2062,12 +2148,15 @@ gossip_group_chat_new (GossipChatroomProvider *provider,
 	g_signal_connect (provider, "chatroom-topic-changed",
 			  G_CALLBACK (group_chat_topic_changed_cb),
 			  chat);
-	g_signal_connect (provider, "chatroom-contact-joined",
+	g_signal_connect (chatroom, "contact-joined",
 			  G_CALLBACK (group_chat_contact_joined_cb),
 			  chat);
-	g_signal_connect (provider, "chatroom-contact-left",
+	g_signal_connect (chatroom, "contact-left",
 			  G_CALLBACK (group_chat_contact_left_cb),
 			  chat);
+	g_signal_connect (chatroom, "contact-info-changed",
+			  G_CALLBACK (group_chat_contact_info_changed_cb),
+			  chat);
 
 	/* Actually join the chat room */
 	group_chat_join (chat);
