@@ -16,8 +16,6 @@
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
- * 
- * Authors: Martyn Russell <martyn@imendio.com>
  */
 
 #include "config.h"
@@ -39,6 +37,7 @@
 
 #include "gossip-app.h"
 #include "gossip-account-chooser.h"
+#include "gossip-glade.h"
 #include "gossip-log-window.h"
 #include "gossip-ui-utils.h"
 
@@ -216,7 +215,9 @@ log_window_find_pixbuf_data_func (GtkCellLayout   *cell_layout,
 
 	session = gossip_app_get_session ();
 	online = gossip_session_is_connected (session, account);
-	pixbuf = gossip_pixbuf_from_account_status (account, GTK_ICON_SIZE_MENU, online);
+	pixbuf = gossip_account_status_create_pixbuf (account, 
+						      GTK_ICON_SIZE_MENU,
+						      online);
 
 	g_object_set (cell, "pixbuf", pixbuf, NULL);
 	g_object_set (cell, "visible", TRUE, NULL);
@@ -1478,7 +1479,6 @@ log_window_chatrooms_get_messages (GossipLogWindow *window,
 	GossipAccount  *account;
 	GossipChatroom *chatroom;
 	GossipContact  *own_contact;
-	GossipContact  *sender;
 	GossipMessage  *message;
 	GList          *messages;
 	GList          *dates = NULL;
@@ -1613,19 +1613,10 @@ log_window_chatrooms_get_messages (GossipLogWindow *window,
 	for (l = messages; l; l = l->next) {
 		message = l->data;
 
-		sender = gossip_message_get_sender (message);
-
-		if (gossip_contact_equal (own_contact, sender)) {
-			gossip_chat_view_append_message_from_self (window->chatview_chatrooms,
-								   message,
-								   own_contact,
-								   NULL);
-		} else {
-			gossip_chat_view_append_message_from_other (window->chatview_chatrooms,
-								    message,
-								    sender,
-								    NULL);
-		}
+		gossip_chat_view_append_message_from_other (window->chatview_chatrooms,
+							    message,
+							    gossip_message_get_sender (message),
+							    NULL);
 	}
 
 	g_list_foreach (messages, (GFunc) g_object_unref, NULL);
