@@ -16,10 +16,6 @@
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
- * 
- * Authors: Mikael Hallendal <micke@imendio.com>
- *          Richard Hult <richard@imendio.com>
- *          Martyn Russell <martyn@imendio.com>
  */
 
 #include "config.h"
@@ -49,11 +45,12 @@
 
 #include "gossip-account-chooser.h"
 #include "gossip-app.h"
-#include "gossip-vcard-dialog.h"
+#include "gossip-glade.h"
 #include "gossip-image-chooser.h"
 #include "gossip-preferences.h"
 #include "gossip-popup-button.h"
 #include "gossip-ui-utils.h"
+#include "gossip-vcard-dialog.h"
 
 #define DEBUG_DOMAIN "VCardDialog"
 
